@@ -46,6 +46,7 @@ OWN_OPTION_SPEC = dict( { 'caption': str,
     'plot_format': str,
     'convert': str,
     'show_source': str,
+    'hidden': str,
     })
 
 class PlotError(SphinxError):
@@ -81,7 +82,7 @@ class PlotDirective(directives.images.Figure):
         if isinstance(node, nodes.system_message):
             return [node]
 
-        node.plot = dict(cmd=cmd,text=text,options=own_options,suffix="plot",
+        node.plot = dict(cmd=cmd,text=text,options=own_options,
                 directive="plot", total_options=total_options)
         return [node]
 
@@ -99,6 +100,10 @@ def render_plot_images(app, doctree):
         try:
             #relfn, outfn, relinfile = cmd_2_image(app, fig.plot)
             out = cmd_2_image(app, fig.plot)
+            if options.get("hidden", False):
+                #Don't render the image if there is hidden
+                nodes.figure.pop(fig)
+                continue
             caption_node = nodes.caption("", options.get("caption", cmd))
             fig += caption_node
             fig['ids'] = "plot"
@@ -132,6 +137,10 @@ def render_plot_images(app, doctree):
         try:
             #relfn, outfn, relinfile = cmd_2_image(app, img.plot)
             out = cmd_2_image(app, img.plot)
+            if options.get("hidden", False):
+                #Don't render the image if there is hidden
+                nodes.image.pop(img)
+                continue
             img['uri'] = out["outrelfn"]
             if out["outreference"]:
                 reference_node = nodes.reference(refuri=out["outreference"])
@@ -145,50 +154,33 @@ def render_plot_images(app, doctree):
             img.replace_self(nodes.literal_block("", "%s\n%s" %(cmd, text)))
             continue
 
-def cmd_2_image (app, plot):
-    """Render plot code into a PNG output file."""
-    #print("app.builder.format: %s" %(app.builder.format))
-    #print("app.builder.env.docname: %s" %(app.builder.env.docname))
-    #print("app.builder.imagedir: %s" %(app.builder.imagedir))
-
+def plot_image (app, plot):
+    '''
+    always plot the image, should check if the target exists before calling me.
+    '''
     cmd = plot['cmd']
     args = shlex.split(cmd)
     text = plot['text']
-    options = plot['options']
-    format = app.builder.format
+    options = plot['total_options']
     rel_imgpath = relative_uri(app.builder.env.docname, app.builder.imagedir)
     hashkey = cmd + str(options) + text
     hashkey = sha(hashkey.encode('utf-8')).hexdigest()
-    infname = '%s-%s.%s' % (args[0], hashkey, plot['suffix'])
+    infname = '%s-%s.%s' % (args[0], hashkey, plot['directive'])
     infullfn = path.join(app.builder.outdir, app.builder.imagedir, infname)
     ensuredir(path.join(app.builder.outdir, app.builder.imagedir))
     currpath = os.getcwd() # Record the current dir and return here afterwards
-
-    if options.get("plot_format", None): #User definition is higher priority.
-        plot_format = options["plot_format"]
-    else:
-        format_map = OUTPUT_DEFAULT_FORMATS.copy()
-        format_map.update(app.builder.config.plot_output_format)
-        plot_format = format_map.get(app.builder.format, "png")
-    # convert only support png.
-    plot_format = options.get("convert", None) and "png" or plot_format
-
+    plot_format = options["plot_format"] # plot_format option must be set before.
     outfname = '%s-%s.%s' %(args[0], hashkey, plot_format)
     out = dict(outrelfn = posixpath.join(rel_imgpath, outfname),
         outfullfn = path.join(app.builder.outdir, app.builder.imagedir, outfname),
-        #outreference = posixpath.join(rel_imgpath, infname),
         outreference = None)
-    #print(out)
 
-    if path.isfile(out["outfullfn"]):
-        print("file has already existed: %s" %(outfname))
-        return out
     if "ditaa" in cmd:
         if plot_format in ["svg", "pdf"]:
             args.insert(1, "--svg") #ditaa support vector image by --svg parameter.
         args.extend([infname, outfname])
         # Ditaa must work on the target directory.
-        os.chdir(path.join(app.builder.outdir, app.builder.imagedir))
+        os.chdir(os.path.dirname(out["outfullfn"]))
     elif args[0] == "dot":
         # dot -Tpng in_file -o out_file
         args.extend([infullfn, '-o', out["outfullfn"]])
@@ -278,14 +270,56 @@ def cmd_2_image (app, plot):
             os.system("inkscape -f %s-%s.svg -A %s"
                     %(args[0], hashkey, out["outfullfn"]))
 
-    # 3) Check if it's need to generate the outreference file
-    if options.get("show_source", False):
-        # Input file could be rendered as a link
-        out["outreference"] = posixpath.join(rel_imgpath, infname)
-
     os.chdir(currpath)
     return out
 
+def cmd_2_image (app, plot):
+    """Render plot code into a PNG output file."""
+    #print("app.builder.format: %s" %(app.builder.format))
+    #print("app.builder.env.docname: %s" %(app.builder.env.docname))
+    #print("app.builder.imagedir: %s" %(app.builder.imagedir))
+
+    cmd = plot['cmd']
+    args = shlex.split(cmd)
+    text = plot['text']
+    options = plot['total_options']
+
+    if options.get("plot_format", None): #User definition is higher priority.
+        plot_format = options["plot_format"]
+    else:
+        format_map = OUTPUT_DEFAULT_FORMATS.copy()
+        format_map.update(app.builder.config.plot_output_format)
+        plot_format = format_map.get(app.builder.format, "png")
+    # convert only support png.
+    plot_format = options.get("convert", None) and "png" or plot_format
+    options["plot_format"] = plot_format
+
+    hashkey = cmd + str(options) + text
+    hashkey = sha(hashkey.encode('utf-8')).hexdigest()
+    outfname = '%s-%s.%s' %(args[0], hashkey, plot_format)
+    rel_imgpath = relative_uri(app.builder.env.docname, app.builder.imagedir)
+    out = dict(outrelfn = posixpath.join(rel_imgpath, outfname),
+        outfullfn = path.join(app.builder.outdir, app.builder.imagedir, outfname),
+        outreference = None)
+    if not path.isfile(out["outfullfn"]):
+        out = plot_image(app, plot) # Really plot the image
+    else:
+        print("file has already existed: %s" %(outfname))
+
+    if options.get("show_source", False):
+        out["outreference"] = posixpath.join(rel_imgpath, infname)
+    if (options.get("name", None) and app.builder.format in ["html"]):
+        #Rename the output to options["name"] only in html
+        outfname = '%s.%s' %(options["name"], plot_format)
+        out2 = dict(outrelfn = posixpath.join(rel_imgpath, outfname),
+            outfullfn = path.join(app.builder.outdir, app.builder.imagedir, outfname),
+            outreference = out["outreference"])
+        shutil.copy(out["outfullfn"], out2["outfullfn"])
+        print("copy %s -> %s" %(os.path.basename(out["outfullfn"]),
+            os.path.basename(out2["outfullfn"])))
+        out = out2
+    return out
+
 def setup(app):
     app.add_directive('plot', PlotDirective)
     app.connect('doctree-read', render_plot_images)
